use bracket_lib::prelude::*;
use specs::prelude::*;

use crate::{components::Position, resources::Map};

pub struct MapIndexingSystem {}

impl<'a> System<'a> for MapIndexingSystem {
    type SystemData = (
        WriteExpect<'a, Map>,
        ReadStorage<'a, Position>,
        Entities<'a>,
    );

    fn run(&mut self, data: Self::SystemData) {
        let (mut map, position, entities) = data;

        map.clear_all_tile_content();
        for (entity, position) in (&entities, &position).join() {
            let index = map.point2d_to_index(Point {
                x: position.x,
                y: position.y,
            });
            map.set_tile_content(index, entity);
        }
    }
}
